-- Nephilist: Garrison
-- By lieandswell <Nephthys @ Hyjal-US>
-- Simple improvements to garrison interface


local THREAT_ICON_SIZE = 28;
local THREAT_ICON_SPACING = 10;

-- Function for testing things
local function NephilistTest(stuff)
	if ( stuff ) then 
		DEFAULT_CHAT_FRAME:AddMessage("Test: "..stuff);
	else 
		DEFAULT_CHAT_FRAME:AddMessage("Test");
	end
end

-- Hide gratuitous "(In Progress)" text from in-progress mission list (srsly wtf)
hooksecurefunc("GarrisonMissionList_Update", function()
	-- Function defined in Blizzard_GarrisonMissionUI.lua

	local self = GarrisonMissionFrame.MissionTab.MissionList;
	local missions;
	if ( self.showInProgress ) then
		missions = self.inProgressMissions;
	else
		missions = self.availableMissions;
	end
	local numMissions = #missions;
	local scrollFrame = self.listScroll;
	local offset      = HybridScrollFrame_GetOffset(scrollFrame);
	local buttons     = scrollFrame.buttons;
	local numButtons  = #buttons;

	for i = 1, numButtons do
		local button = buttons[i];
		local index  = offset + i; -- adjust index for scrolling
		if ( index <= numMissions ) then
			local mission = missions[index];
			if ( mission.inProgress ) then
				button.Summary:SetText("("..mission.timeLeft..")");
			end
		end
	end

end)

-- Sort in-progress missions by time remaining
local function SecondsLeft(timeLeft) 
	local secondsLeft = 0;
	local hrLeft,  _ = string.match(timeLeft, "(%d+)( hr)")
	local minLeft, _ = string.match(timeLeft, "(%d+)( min)")
	local secLeft, _ = string.match(timeLeft, "(%d+)( sec)")
	if ( hrLeft ) then
		secondsLeft = secondsLeft + 60 * 60 * tonumber(hrLeft);
	end
	if ( minLeft ) then
		secondsLeft = secondsLeft + 60 * tonumber(minLeft);
	end
	if ( secLeft ) then
		secondsLeft = secondsLeft + tonumber(secLeft);
	end
	return secondsLeft
end
local function SortInProgressMissions(missionsList)
	local comparison = function(mission1, mission2)

		secondsLeft1 = SecondsLeft(mission1.timeLeft);
		secondsLeft2 = SecondsLeft(mission2.timeLeft);
		if ( secondsLeft1 ~= secondsLeft2 ) then
			return secondsLeft1 < secondsLeft2;
		end

		-- Default Blizzard sort below here, since timeLeft doesn't always include seconds
		if ( mission1.level ~= mission2.level ) then
			return mission1.level > mission2.level;
		end
		if ( mission1.level == GARRISON_FOLLOWER_MAX_LEVEL ) then
			if ( mission1.iLevel ~= mission2.iLevel ) then
				return mission1.iLevel > mission2.iLevel;
			end	
		end
		if ( mission1.durationSeconds ~= mission2.durationSeconds ) then
			return mission1.durationSeconds < mission2.durationSeconds;
		end
		if ( mission1.isRare ~= mission2.isRare ) then
			return mission1.isRare;
		end
		return strcmputf8i(mission1.name, mission2.name) < 0;
	end
	table.sort(missionsList, comparison);
end
hooksecurefunc(C_Garrison, "GetInProgressMissions", function(missionTable)
	-- C_Garrison.GetInProgressMissions is part of the API
	if ( missionTable ) then SortInProgressMissions(missionTable) end
end)

-- Show time remaining for followers on missions
hooksecurefunc("GarrisonFollowerList_Update", function(self)
	-- Function defined in Blizzard_GarrisonSharedTemplates.lua
	-- self is GarrisonMissionFrameFollowers, defined in Blizzard_GarrisonMissionUI.xml

	local followerFrame = self;
	local followers     = followerFrame.FollowerList.followers;
	local followersList = followerFrame.FollowerList.followersList;
	local scrollFrame   = followerFrame.FollowerList.listScroll;
	local offset        = HybridScrollFrame_GetOffset(scrollFrame);
	local buttons       = scrollFrame.buttons;

	for i = 1, #buttons do
		local button = buttons[i];
		local index  = offset + i; -- adjust for scrolling
		if ( index <= #followersList ) then
			local follower = followers[followersList[index]];
			if ( follower.status == GARRISON_FOLLOWER_ON_MISSION ) then
				for k, mission in pairs(C_Garrison.GetInProgressMissions()) do
					for i = 1, #mission.followers do
						if ( mission.followers[i] == follower.followerID ) then
							button.Status:SetFormattedText("%s", mission.timeLeft);
						end
					end
				end
			end
		end
	end

end)

-- Show follower counters even if they're working or away on a mission
local function ShowMoreCounters(button, follower, showCounters) 
	local numShown = 0;
	if ( showCounters and button.isCollected ) then
		-- if a mission is being viewed, show mechanics this follower can counter
		-- for followers you have, show counters if they are or could be on the mission
		local counters = GarrisonMissionFrame.followerCounters and GarrisonMissionFrame.followerCounters[follower.followerID];
		if ( counters ) then
			for i = 1, #counters do
				-- max of 4 icons
				if ( numShown == 4 ) then
					break;
				end			
				numShown = numShown + 1;
				GarrisonFollowerButton_SetCounterButton(button, numShown, counters[i]);
			end
		end
		local traits = GarrisonMissionFrame.followerTraits and GarrisonMissionFrame.followerTraits[follower.followerID];
		if ( traits ) then
			for i = 1, #traits do
				-- max of 4 icons
				if ( numShown == 4 ) then
					break;
				end
				numShown = numShown + 1;
				GarrisonFollowerButton_SetCounterButton(button, numShown, traits[i]);
			end
		end
	end
	if ( numShown == 1 or numShown == 2 ) then
		button.Counters[1]:SetPoint("TOPRIGHT", -8, -16);
	else
		button.Counters[1]:SetPoint("TOPRIGHT", -8, -4);
	end
	for i = numShown + 1, #button.Counters do
		button.Counters[i].info = nil;
		button.Counters[i]:Hide();
	end
end
hooksecurefunc("GarrisonFollowerButton_UpdateCounters", ShowMoreCounters)
	-- Function defined in Blizzard_GarrisonSharedTemplates.lua

-- Show enemy threats in mission list
-- Fade out threats that can't be currently countered
-- Show bonus xp in reward icon
local function ShowMissionThreats(self, rewards, numRewards) 
	-- self here is the mission button (GarrisonMissionListButtonTemplate)

	if ( not self.info.inProgress ) then

		local _, xp, _, _, _, _, _, enemies = C_Garrison.GetMissionInfo(self.info.missionID);
		local counters                      = C_Garrison.GetBuffedFollowersForMission(self.info.missionID)
		local followers                     = C_Garrison.GetFollowers();

		-- Frame to hold threat icons
		local nephilistThreats = self.NephilistThreatsFrame;
		if ( not nephilistThreats ) then
			nephilistThreats = CreateFrame("Frame", nil, self);
			nephilistThreats:SetPoint("CENTER", self, "RIGHT", -215, 0); 
			nephilistThreats:SetWidth(0);
			nephilistThreats:SetHeight(26);
			nephilistThreats.Threats = {};
			-- nephilistThreats.Threats[1] = CreateFrame("Frame", nil, nephilistThreats, "GarrisonAbilityCounterTemplate");
			nephilistThreats.Threats[1] = CreateFrame("Frame", nil, nephilistThreats, "GarrisonMissionLargeMechanicTemplate");
			nephilistThreats.Threats[1]:SetPoint("LEFT");
			self.NephilistThreatsFrame = nephilistThreats;
		end

		-- Show mission threats
		local numThreats = 0;
		for i = 1, #enemies do
			local enemy = enemies[i];
			for id, mechanic in pairs(enemy.mechanics) do

				numThreats = numThreats + 1;
				local threatFrame = nephilistThreats.Threats[numThreats];
				if ( not threatFrame ) then
					threatFrame = CreateFrame("Frame", nil, nephilistThreats, "GarrisonMissionLargeMechanicTemplate");
					-- threatFrame:SetPoint("LEFT", nephilistThreats.Threats[numThreats - 1], "RIGHT", THREAT_ICON_SPACING, 0);
					tinsert(nephilistThreats.Threats, threatFrame);
				end
				threatFrame.Icon:SetTexture(mechanic.icon);
				threatFrame:Show();

				-- Fade out threat icons with no current counters
				threatFrame.counterFound = false;
				if ( counters ) then
					for i = 1, #followers do
						local follower = followers[i];
						-- local status = C_Garrison.GetFollowerStatus(follower.followerID);
						-- Don't know why that doesn't work
						if ( follower.isCollected ) 
							and ( counters[follower.followerID] ) 
							and not ( C_Garrison.GetFollowerStatus(follower.followerID) == GARRISON_FOLLOWER_ON_MISSION ) 
							and not ( C_Garrison.GetFollowerStatus(follower.followerID) == GARRISON_FOLLOWER_INACTIVE )  
						then
							local counter = counters[follower.followerID];
							for k = 1, #counter do 
								if ( counter[k].name == mechanic.name ) then
									threatFrame.counterFound = true;
								end
							end
						end
					end
				end
				if ( threatFrame.counterFound ) then 
					threatFrame:SetAlpha(1); 
				else 
					threatFrame:SetAlpha(1/3); 
				end

			end
		end

		-- Hide icons that were made by previous missions in that button
		for i = numThreats + 1, #nephilistThreats.Threats do
			nephilistThreats.Threats[i]:Hide(); 
		end

		-- Arrange icons
		if ( numThreats > 4 ) then 
			-- Not yet implemented. Max is six, right? 
			nephilistThreats.Threats[1]:SetPoint("LEFT");
			for i = 2, numThreats do 
				nephilistThreats.Threats[i]:SetPoint("LEFT", nephilistThreats.Threats[i - 1], "RIGHT", THREAT_ICON_SPACING, 0);
			end
			nephilistThreats:SetWidth(numThreats * (THREAT_ICON_SIZE + THREAT_ICON_SPACING));
			nephilistThreats:SetHeight(THREAT_ICON_SIZE);
		else
			nephilistThreats.Threats[1]:SetPoint("LEFT");
			for i = 2, numThreats do 
				nephilistThreats.Threats[i]:SetPoint("LEFT", nephilistThreats.Threats[i - 1], "RIGHT", THREAT_ICON_SPACING, 0);
			end
			nephilistThreats:SetWidth(numThreats * (THREAT_ICON_SIZE + THREAT_ICON_SPACING));
			nephilistThreats:SetHeight(THREAT_ICON_SIZE);
		end

		if ( numThreats > 0 ) then
			nephilistThreats:Show();
		else
			nephilistThreats:Hide();
		end

	else -- mission is in progress

		-- Hide threats for missions in progress
		if ( self.NephilistThreatsFrame ) then
			self.NephilistThreatsFrame:Hide();
		end	

	end

	-- Show bonus experience on reward icon
	local i = 1;
	for key, reward in pairs(rewards) do
		local button = self.Rewards[i]; 
		if ( reward.followerXP ) then
			button.Quantity:SetText(reward.followerXP);
			button.Quantity:Show();
		end
		i = i + 1;
	end

end
hooksecurefunc("GarrisonMissionButton_SetRewards", ShowMissionThreats);
	-- Function defined in Blizzard_GarrisonMissionUI.lua

-- Filter follower list by clicking threat icon in mission pane
local function MechanicFrame_OnMouseUp(self)
	local searchBox = GarrisonMissionFrameFollowers.SearchBox;
	searchBox:SetText(self.info.name);
	SearchBoxTemplate_OnTextChanged(searchBox);
	GarrisonFollowerList_UpdateFollowers(searchBox:GetParent());
end
hooksecurefunc("GarrisonMissionPage_SetEnemies", function(enemies, numFollowers)
	-- Function in defined in Blizzard_GarrisonMissionUI.lua
	for i=1, #enemies do
		local Frame =  GarrisonMissionFrame.MissionTab.MissionPage.Enemies[i];
		if ( not Frame ) then
			break;
		end	
		local enemy = enemies[i];
		local numMechs = 0;
		for id, mechanic in pairs(enemy.mechanics) do
			numMechs = numMechs + 1;
			local mechanicFrame = Frame.Mechanics[numMechs];
			mechanicFrame:SetScript("OnMouseUp", MechanicFrame_OnMouseUp); 
		end
	end
end)


